/**
  * @file BrushlessPIC32.c
  * @brief This module contains the implementation of the brushless hardware
  * functions.

  * @author Stefan Popovic
  *
  * @date 11/09/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: P32MX795F512L
  * Dev.Board: EasyPIC Fusion v7
  * Oscillator: HS-PLL 80 MHz
  * Ext. Modules: Brushless Click Board
  * SW: MikroC PRO for PIC32 v3.6.0

  * NOTES:
    - Place Brushless click board on mikroBUS slot 1.
    - Turn on PORTD LEDs at SW15, pull-down PORTB.0 and PORTB.1 and turn on Button Press Level switch SW10 for PORTB/L. (board specific)
    - PWM: RD0  Channel 1, prescaler 1
    - Vsense: RB8
    - Int: PE8

  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/
  #define CW    0x01
  #define CCW   0x00
  #define STEP  100
  
  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/
  
  /**************************************************************************
  * Includes
  **************************************************************************/
  #include "built_in.h"
  #include "recources.h"
  
  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/
  sbit MOTOR_DIR at RC1_bit;

  unsigned int current_speed;
  unsigned int pwm_period;
  char txt[24];

  // TFT module connections
  char TFT_DataPort at LATE;
  sbit TFT_RST at LATD7_bit;
  sbit TFT_BLED at LATD2_bit;
  sbit TFT_RS at LATD9_bit;
  sbit TFT_CS at LATD10_bit;
  sbit TFT_RD at LATD5_bit;
  sbit TFT_WR at LATD4_bit;
  char TFT_DataPort_Direction at TRISE;
  sbit TFT_RST_Direction at TRISD7_bit;
  sbit TFT_BLED_Direction at TRISD2_bit;
  sbit TFT_RS_Direction at TRISD9_bit;
  sbit TFT_CS_Direction at TRISD10_bit;
  sbit TFT_RD_Direction at TRISD5_bit;
  sbit TFT_WR_Direction at TRISD4_bit;
  // End TFT module connections

  void Display_Init()
  {
    TFT_BLED_Direction = 0;
    TFT_Init_ILI9341_8bit(320, 240);
    TFT_BLED = 1;

   }

  void MCU_Init() 
  {
    TRISF0_bit = 1;                            //configure PORTF pin 0 as input
    TRISF1_bit = 1;                            //configure PORTF pin 1 as input
    TRISC1_bit = 0;                            //configure PORTC pin 1 as output (VSense)
    JTAGEN_bit = 0;

    AD1PCFG = 0xFFFF;                          // Configure AN pins as digital I/O
    TRISB = 0xFFEF;                            // configure PORTB pins as input
    PORTD = 0;                                 // set PORTD to 0
    TRISD = 0;                                 // designate PORTD pins as output
    ADC1_Init();                               // ADC Init
    Delay_ms(100);
    current_speed = 3000;                     // initial value for current duty

    pwm_period = PWM_Init(5000 , 1, 1, 2);     // PWM Initialization
    PWM_Start(1);
    PWM_Set_Duty(current_speed,  1);           // Set current duty for PWM1
   }

   void DrawFrame()
   {
    TFT_Fill_Screen(CL_TEAL);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_TEAL, 0, 0, CL_TEAL, CL_TEAL);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Brushless  Click  Board  Example", 12, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyPIC Fusion v7 for P32MX", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("PWM  DUTY", 10, 80 );
    TFT_Write_Text("PWM  MAX", 10, 140);
    WordToStr(pwm_period, txt);
    TFT_Write_Text(txt, 150, 140);
    TFT_Set_Pen(CL_TEAL, 1);
  }

  void main() {
    MCU_Init();
    TFT_BLED_Direction = 0;
    TFT_Init_ILI9341_8bit(320, 240); //TFT Init
    TFT_BLED = 1;
    DrawFrame();
    MOTOR_DIR = CCW;  // Setting motor direction to counter-clock-wise
    
    while(1) {

          if (RB0_bit)                            // button on RB0 pressed
          {
            Delay_ms(30);
            current_speed = current_speed + STEP;      // increment current_duty
            if (current_speed > pwm_period)       // if we increase current_duty greater then possible pwm_period1 value
            {
             current_speed = 0;                    // reset current_duty value to zero
            }
            PWM_Set_Duty(current_speed,  1);        // set newly acquired duty ratio
         }

        if (RB1_bit)                            // button on RB1 pressed
        {
            Delay_ms(30);
            current_speed = current_speed - STEP;       // decrement current duty
            if (current_speed > pwm_period)       // if we decrease current duty greater then possible pwm period value
            {
              current_speed = pwm_period;          // set current_duty to max possible value
            }
            PWM_Set_Duty(current_speed,  1);        // set newly acquired duty ratio
         }

        Delay_ms(20);

        // current_speed display
        WordToStr(current_speed, txt);
        TFT_Rectangle(150, 80, 250, 110);
        TFT_Write_Text(txt, 150, 80);
    }
  }